@@ -4,6 +4,7 @@ import types
 import grpc
 import logging
 from contextlib import asynccontextmanager
+import itertools
 
 import HStream.Server.HStreamApi_pb2 as ApiPb
 import HStream.Server.HStreamApi_pb2_grpc as ApiGrpc
@@ -23,9 +24,10 @@ from hstreamdb.types import (
     ShardOffset,
 )
 from hstreamdb.utils import (
-    cons_record,
     find_shard_id,
-    parse_recived_records,
+    decode_records,
+    encode_records,
+    encode_records_from_append_payload,
 )
 
 __all__ = ["insecure_client", "HStreamDBClient"]
@@ -50,23 +52,27 @@ def dec_api(f):
 
 
 class HStreamDBClient:
-    _stub: ApiGrpc.HStreamApiStub
-
     _TargetTy = str
 
-    _channels: Dict[_TargetTy, Optional[grpc.aio.Channel]] = {}
+    _stub: ApiGrpc.HStreamApiStub
+    _channels: Dict[_TargetTy, Optional[grpc.aio.Channel]]
 
     _current_target: _TargetTy
     # {(stream_name, shard_id)}
-    _append_channels: Dict[Tuple[str, int], _TargetTy] = {}
-    _subscription_channels: Dict[str, _TargetTy] = {}
-    _reader_channels: Dict[str, _TargetTy] = {}
-    _shards_info: Dict[str, List[Shard]] = {}
+    _append_channels: Dict[Tuple[str, int], _TargetTy]
+    _subscription_channels: Dict[str, _TargetTy]
+    _reader_channels: Dict[str, _TargetTy]
+    _shards_info: Dict[str, List[Shard]]
 
     _cons_target = staticmethod(lambda host, port: f"{host}:{port}")
 
     def __init__(self, host: str = "127.0.0.1", port: int = 6570):
         self._current_target = self._cons_target(host, port)
+        self._channels = {}
+        self._append_channels = {}
+        self._subscription_channels = {}
+        self._reader_channels = {}
+        self._shards_info = {}
         # TODO: secure_channel
         _channel = grpc.aio.insecure_channel(self._current_target)
         self._channels[self._current_target] = _channel
@@ -122,6 +128,8 @@ class HStreamDBClient:
         name: str,
         payloads: Iterable[Any],
         key: Optional[str] = None,
+        compresstype=None,
+        compresslevel=9,
     ) -> Iterator[RecordId]:
         """Append payloads to a stream.
 
@@ -133,27 +141,32 @@ class HStreamDBClient:
         Returns:
             Appended RecordIds generator
         """
-
         shard_id, channel = await self._lookup_append(name, key, None)
         stub = ApiGrpc.HStreamApiStub(channel)
         r = await stub.Append(
             ApiPb.AppendRequest(
                 streamName=name,
                 shardId=shard_id,
-                records=map(lambda p: cons_record(p, key), payloads),
+                records=encode_records(
+                    payloads,
+                    key=key,
+                    compresstype=compresstype,
+                    compresslevel=compresslevel,
+                ),
             )
         )
-
         return (record_id_from(x) for x in r.recordIds)
 
     def new_producer(
         self,
         append_callback: Optional[Type[BufferedProducer.AppendCallback]] = None,
-        size_trigger=0,
+        size_trigger=0,  # NOTE: this is the size of uncompressed records
         time_trigger=0,
         workers=1,
         retry_count=0,
         retry_max_delay=60,
+        compresstype=None,
+        compresslevel=9,
     ):
         return BufferedProducer(
             self._append_with_shard,
@@ -167,7 +180,7 @@ class HStreamDBClient:
         )
 
     @dec_api
-    async def list_shards(self, stream_name) -> [Shard]:
+    async def list_shards(self, stream_name) -> List[Shard]:
         # FIXME: what if shards_info can be changed?
         shards = self._shards_info.get(stream_name)
         if not shards:
@@ -289,11 +302,13 @@ class HStreamDBClient:
     ) -> Iterator[Record]:
         channel = await self._lookup_reader(reader_id)
         stub = ApiGrpc.HStreamApiStub(channel)
-        r = await stub.ReadShard(
+        resp = await stub.ReadShard(
             ApiPb.ReadShardRequest(readerId=reader_id, maxRecords=max_records)
         )
 
-        return parse_recived_records(r.receivedRecords)
+        return itertools.chain.from_iterable(
+            decode_records(r) for r in resp.receivedRecords
+        )
 
     @dec_api
     async def delete_reader(self, reader_id: str) -> None:
@@ -313,14 +328,23 @@ class HStreamDBClient:
         name: str,
         payloads: List[AppendPayload],
         shard_id: int,
+        compresstype=None,
+        compresslevel=9,
     ) -> Iterator[RecordId]:
+        if not payloads:
+            logger.warning("Empty payloads, ignored.")
+            return
         shard_id, channel = await self._lookup_append(name, None, shard_id)
         stub = ApiGrpc.HStreamApiStub(channel)
         r = await stub.Append(
             ApiPb.AppendRequest(
                 streamName=name,
                 shardId=shard_id,
-                records=map(lambda p: cons_record(p.payload, p.key), payloads),
+                records=encode_records_from_append_payload(
+                    payloads,
+                    compresstype=compresstype,
+                    compresslevel=compresslevel,
+                ),
             )
         )
 
