@@ -11,10 +11,14 @@ from typing import (
     Dict,
     Sized,
     Tuple,
+    Union,
 )
 import logging
 
+import HStream.Server.HStreamApi_pb2 as ApiPb
+
 from hstreamdb.types import RecordId
+from hstreamdb.utils import encode_payload
 
 __all__ = [
     "PayloadsFull",
@@ -28,14 +32,12 @@ logger = logging.getLogger(__name__)
 
 
 class Timer:
-    _continue: asyncio.Event = asyncio.Event()
-    _enable: bool = True
-    _task: asyncio.Task
-
     def __init__(self, delay, coro):
         self._delay = delay
         self._coro = coro
-        self._task = asyncio.create_task(self._loop())
+        self._continue: asyncio.Event = asyncio.Event()
+        self._enable: bool = True
+        self._task: asyncio.Task = asyncio.create_task(self._loop())
 
     def start(self):
         self._continue.set()
@@ -75,26 +77,20 @@ class Payload(abc.ABC, Sized):
 
 
 class PayloadGroup:
-    _payloads: List[Payload] = []
-    _size: int = 0
-    _lock: asyncio.Lock
-
-    _maxsize: int
-    _timer: Optional[Timer] = None
-
-    _flushing_payloads: List[Payload] = []
-    _flushing_size: int = 0
-    _flush_done: asyncio.Event = asyncio.Event()
-    _key: Any
-    _notify_queue: asyncio.Queue
-
     def __init__(self, queue, key, maxsize=0, maxtime=0):
-        self._key = key
-        self._notify_queue = queue
-        self._lock = asyncio.Lock()
-        self._maxsize = maxsize
-        if maxtime > 0:
-            self._timer = Timer(maxtime, self.flush)
+        self._payloads: List[Payload] = []
+        self._size: int = 0
+        self._flushing_payloads: List[Payload] = []
+        self._flushing_size: int = 0
+        self._flush_done: asyncio.Event = asyncio.Event()
+
+        self._key: Any = key
+        self._notify_queue: asyncio.Queue = queue
+        self._lock: asyncio.Lock = asyncio.Lock()
+        self._maxsize: int = maxsize
+        self._timer: Optional[Timer] = (
+            Timer(maxtime, self.flush) if maxtime > 0 else None
+        )
 
     async def append(self, payload: Payload):
         if not self._payloads and self._timer:
@@ -174,26 +170,26 @@ class PayloadGroup:
 
 
 class AppendPayload(Payload):
-    # TODO: support dict
-    #
-    # Typically payload can be bytes, str, or dict. But for now, dict is unsupported
-    payload: bytes
-    key: Optional[str]
+    _payload_bin: bytes
+    _payload_type: ApiPb.HStreamRecordHeader.Flag
 
-    def __init__(self, payload: bytes, key: Optional[str] = None):
-        self.payload = payload
-        self.key = key
+    def __init__(
+        self,
+        payload: Union[bytes, str, Dict[Any, Any]],
+        key: Optional[str] = None,
+    ):
+        self.payload: Union[bytes, str, Dict[Any, Any]] = payload
+        self._payload_bin, self._payload_type = encode_payload(self.payload)
+        self.key: Optional[str] = key
 
     def __len__(self):
-        return len(self.payload)
+        return len(self._payload_bin)
 
 
 class BufferedProducer:
     StreamKeyId = int
     GroupKeyTy = Tuple[str, StreamKeyId]  # (stream_name, shard_id)
 
-    _group: Dict[GroupKeyTy, PayloadGroup] = {}
-
     class AppendCallback(abc.ABC):
         @abc.abstractmethod
         def on_success(
@@ -217,7 +213,8 @@ class BufferedProducer:
     def __init__(
         self,
         flush_coro: Callable[
-            [str, List[AppendPayload], int], Awaitable[Iterator[RecordId]]
+            [str, List[AppendPayload], int, Optional[str], int],
+            Awaitable[Iterator[RecordId]],
         ],
         find_stream_key_id_coro: Callable[
             [str, Optional[str]], Awaitable[StreamKeyId]
@@ -228,14 +225,19 @@ class BufferedProducer:
         workers=1,
         retry_count=0,
         retry_max_delay=60,  # seconds
+        compresstype=None,
+        compresslevel=9,
     ):
         if workers < 1:
             raise ValueError("workers must be no less than 1")
+        self._group: Dict[BufferedProducer.GroupKeyTy, PayloadGroup] = {}
         self._size_trigger = size_trigger
         self._time_trigger = time_trigger
         self._retry_count = retry_count
         self._retry_max_delay = retry_max_delay
         self._flush_coro = flush_coro
+        self._compresstype = compresstype
+        self._compresslevel = compresslevel
         self._find_stream_key_id_coro = find_stream_key_id_coro
         self._append_callback = append_callback
         self._queues = [asyncio.Queue() for _ in range(workers)]
@@ -245,7 +247,10 @@ class BufferedProducer:
         ]
 
     async def append(
-        self, stream_name: str, payload: bytes, key: Optional[str] = None
+        self,
+        stream_name: str,
+        payload: Union[bytes, str, Dict[Any, Any]],
+        key: Optional[str] = None,
     ):
         group_key = await self._fetch_group_key(stream_name, key)
         bpayload = AppendPayload(payload, key=key)
@@ -266,11 +271,11 @@ class BufferedProducer:
 
     async def flush(self, stream_name: str, shard_id: int):
         group_key = self._cons_group_key(stream_name, shard_id)
-        self._flush(group_key)
+        await self._flush(group_key)
 
     async def flush_by_key(self, stream_name: str, key: Optional[str] = None):
         group_key = await self._fetch_group_key(stream_name, key)
-        self._flush(group_key)
+        await self._flush(group_key)
 
     async def flushall(self):
         for _, payloads in self._group.items():
@@ -327,7 +332,13 @@ class BufferedProducer:
         retries = 0
         while True:
             try:
-                await self._flush_coro(stream_name, payloads, stream_keyid)
+                await self._flush_coro(
+                    stream_name,
+                    payloads,
+                    stream_keyid,
+                    compresstype=self._compresstype,
+                    compresslevel=self._compresslevel,
+                )
                 await payload_group.post_flush()
             except Exception as e:  # TODO: should be a specific append exception
                 if self._retry_count < 0 or retries < self._retry_count:
